@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2010 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -55,10 +55,17 @@ static PrefParam param[] = {
 	/* Send */
 	{"save_message", "TRUE", &prefs_common.savemsg, P_BOOL},
 	{"filter_sent_message", "FALSE", &prefs_common.filter_sent, P_BOOL},
+	{"recipients_autoreg", "TRUE", &prefs_common.recipients_autoreg,
+	 P_BOOL},
 
 	{"encoding_method", "0", &prefs_common.encoding_method, P_ENUM},
 	{"mime_filename_encoding_method", "0",
 	 &prefs_common.mime_fencoding_method, P_ENUM},
+	{"check_attach", "FALSE", &prefs_common.check_attach, P_BOOL},
+	{"check_attach_str", NULL, &prefs_common.check_attach_str, P_STRING},
+	{"check_recipients", "FALSE", &prefs_common.check_recipients, P_BOOL},
+	{"check_recp_exclude", NULL, &prefs_common.check_recp_exclude,
+	 P_STRING},
 
 	{"allow_jisx0201_kana", "FALSE", &prefs_common.allow_jisx0201_kana,
 	 P_BOOL},
@@ -85,6 +92,8 @@ static PrefParam param[] = {
 	 P_BOOL},
 	{"inherit_recipient_on_self_reply", "FALSE",
 	 &prefs_common.inherit_recipient_on_self_reply, P_BOOL},
+	{"reply_address_only", "FALSE",
+	 &prefs_common.reply_address_only, P_BOOL},
 
 	{"show_ruler", "TRUE", &prefs_common.show_ruler, P_BOOL},
 
@@ -112,6 +121,12 @@ static PrefParam param[] = {
 	 &prefs_common.display_folder_unread, P_BOOL},
 	{"display_folder_num_columns", "FALSE",
 	 &prefs_common.display_folder_num_columns, P_BOOL},
+	{"folder_col_show_new", "FALSE",
+	 &prefs_common.folder_col_visible[1], P_BOOL},
+	{"folder_col_show_unread", "FALSE",
+	 &prefs_common.folder_col_visible[2], P_BOOL},
+	{"folder_col_show_total", "FALSE",
+	 &prefs_common.folder_col_visible[3], P_BOOL},
 
 	{"newsgroup_abbrev_len", "16", &prefs_common.ng_abbrev_len, P_INT},
 
@@ -126,6 +141,9 @@ static PrefParam param[] = {
 	{"enable_rules_hint", "TRUE", &prefs_common.enable_rules_hint, P_BOOL},
 	{"bold_unread", "TRUE", &prefs_common.bold_unread, P_BOOL},
 
+	{"persist_qsearch_filter", "TRUE", &prefs_common.persist_qsearch_filter,
+	 P_BOOL},
+
 	{"toolbar_style", "4", &prefs_common.toolbar_style, P_ENUM},
 	{"show_searchbar", "TRUE", &prefs_common.show_searchbar, P_BOOL},
 	{"show_statusbar", "TRUE", &prefs_common.show_statusbar, P_BOOL},
@@ -135,6 +153,8 @@ static PrefParam param[] = {
 	{"compose_toolbar_setting", NULL,
 	 &prefs_common.compose_toolbar_setting, P_STRING},
 
+	{"layout_type", "0", &prefs_common.layout_type, P_ENUM},
+
 	{"summary_col_show_mark", "TRUE",
 	 &prefs_common.summary_col_visible[S_COL_MARK], P_BOOL},
 	{"summary_col_show_unread", "TRUE",
@@ -151,6 +171,8 @@ static PrefParam param[] = {
 	 &prefs_common.summary_col_visible[S_COL_SIZE], P_BOOL},
 	{"summary_col_show_number", "FALSE",
 	 &prefs_common.summary_col_visible[S_COL_NUMBER], P_BOOL},
+	{"summary_col_show_to", "FALSE",
+	 &prefs_common.summary_col_visible[S_COL_TO], P_BOOL},
 
 	{"summary_col_pos_mark", "0",
 	  &prefs_common.summary_col_pos[S_COL_MARK], P_INT},
@@ -168,6 +190,46 @@ static PrefParam param[] = {
 	  &prefs_common.summary_col_pos[S_COL_SIZE], P_INT},
 	{"summary_col_pos_number", "7",
 	  &prefs_common.summary_col_pos[S_COL_NUMBER], P_INT},
+	{"summary_col_pos_to", "8",
+	  &prefs_common.summary_col_pos[S_COL_TO], P_INT},
+
+	{"summary_sent_col_show_mark", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_MARK], P_BOOL},
+	{"summary_sent_col_show_unread", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_UNREAD], P_BOOL},
+	{"summary_sent_col_show_mime", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_MIME], P_BOOL},
+	{"summary_sent_col_show_subject", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_SUBJECT], P_BOOL},
+	{"summary_sent_col_show_from", "FALSE",
+	 &prefs_common.summary_sent_col_visible[S_COL_FROM], P_BOOL},
+	{"summary_sent_col_show_date", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_DATE], P_BOOL},
+	{"summary_sent_col_show_size", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_SIZE], P_BOOL},
+	{"summary_sent_col_show_number", "FALSE",
+	 &prefs_common.summary_sent_col_visible[S_COL_NUMBER], P_BOOL},
+	{"summary_sent_col_show_to", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_TO], P_BOOL},
+
+	{"summary_sent_col_pos_mark", "0",
+	  &prefs_common.summary_sent_col_pos[S_COL_MARK], P_INT},
+	{"summary_sent_col_pos_unread", "1",
+	  &prefs_common.summary_sent_col_pos[S_COL_UNREAD], P_INT},
+	{"summary_sent_col_pos_mime", "2",
+	  &prefs_common.summary_sent_col_pos[S_COL_MIME], P_INT},
+	{"summary_sent_col_pos_subject", "3",
+	  &prefs_common.summary_sent_col_pos[S_COL_SUBJECT], P_INT},
+	{"summary_sent_col_pos_from", "8",
+	  &prefs_common.summary_sent_col_pos[S_COL_FROM], P_INT},
+	{"summary_sent_col_pos_date", "5",
+	  &prefs_common.summary_sent_col_pos[S_COL_DATE], P_INT},
+	{"summary_sent_col_pos_size", "6",
+	  &prefs_common.summary_sent_col_pos[S_COL_SIZE], P_INT},
+	{"summary_sent_col_pos_number", "7",
+	  &prefs_common.summary_sent_col_pos[S_COL_NUMBER], P_INT},
+	{"summary_sent_col_pos_to", "4",
+	  &prefs_common.summary_sent_col_pos[S_COL_TO], P_INT},
 
 	{"summary_col_size_mark", "10",
 	 &prefs_common.summary_col_size[S_COL_MARK], P_INT},
@@ -185,6 +247,8 @@ static PrefParam param[] = {
 	 &prefs_common.summary_col_size[S_COL_SIZE], P_INT},
 	{"summary_col_size_number", "40",
 	 &prefs_common.summary_col_size[S_COL_NUMBER], P_INT},
+	{"summary_col_size_to", "120",
+	 &prefs_common.summary_col_size[S_COL_TO], P_INT},
 
 	/* Widget size */
 	{"folderwin_x", "16", &prefs_common.folderwin_x, P_INT},
@@ -201,11 +265,16 @@ static PrefParam param[] = {
 
 	{"summaryview_width", "600", &prefs_common.summaryview_width, P_INT},
 	{"summaryview_height", "180", &prefs_common.summaryview_height, P_INT},
+	{"summaryview_vwidth", "300", &prefs_common.summaryview_vwidth, P_INT},
+	{"summaryview_vheight", "600", &prefs_common.summaryview_vheight,
+	 P_INT},
 
 	{"main_messagewin_x", "256", &prefs_common.main_msgwin_x, P_INT},
 	{"main_messagewin_y", "210", &prefs_common.main_msgwin_y, P_INT},
 	{"messageview_width", "600", &prefs_common.msgview_width, P_INT},
 	{"messageview_height", "300", &prefs_common.msgview_height, P_INT},
+	{"messageview_vwidth", "300", &prefs_common.msgview_vwidth, P_INT},
+	{"messageview_vheight", "600", &prefs_common.msgview_vheight, P_INT},
 	{"messageview_visible", "TRUE", &prefs_common.msgview_visible, P_BOOL},
 
 	{"mainview_x", "64", &prefs_common.mainview_x, P_INT},
@@ -223,10 +292,14 @@ static PrefParam param[] = {
 
 	{"sourcewin_width", "600", &prefs_common.sourcewin_width, P_INT},
 	{"sourcewin_height", "500", &prefs_common.sourcewin_height, P_INT},
+
 	{"compose_x", "32", &prefs_common.compose_x, P_INT},
 	{"compose_y", "32", &prefs_common.compose_y, P_INT},
 	{"compose_width", "600", &prefs_common.compose_width, P_INT},
 	{"compose_height", "560", &prefs_common.compose_height, P_INT},
+
+	{"compose_maximized", "FALSE", &prefs_common.compose_maximized, P_BOOL},
+
 	{"addressbook_x", "32", &prefs_common.addressbook_x, P_INT},
 	{"addressbook_y", "32", &prefs_common.addressbook_y, P_INT},
 	{"addressbook_width", "620", &prefs_common.addressbook_width, P_INT},
@@ -248,6 +321,8 @@ static PrefParam param[] = {
 	 P_BOOL},
 	{"display_header", "TRUE", &prefs_common.display_header, P_BOOL},
 	{"render_html", "TRUE", &prefs_common.render_html, P_BOOL},
+	{"html_only_as_attach", "FALSE", &prefs_common.html_only_as_attach,
+	 P_BOOL},
 	{"line_space", "2", &prefs_common.line_space, P_INT},
 
 	{"textview_cursor_visible", "FALSE",
@@ -281,16 +356,16 @@ static PrefParam param[] = {
 	/* Junk mail */
 	{"enable_junk", "FALSE", &prefs_common.enable_junk, P_BOOL},
 #ifdef G_OS_WIN32
-	{"junk_learn_command", "bsfilterw -su", &prefs_common.junk_learncmd,
-	 P_STRING},
-	{"nojunk_learn_command", "bsfilterw -cu",
+	{"junk_learn_command", "bsfilterw -C -s -u",
+	 &prefs_common.junk_learncmd, P_STRING},
+	{"nojunk_learn_command", "bsfilterw -c -S -u",
 	 &prefs_common.nojunk_learncmd, P_STRING},
 	{"junk_classify_command", "bsfilterw",
 	 &prefs_common.junk_classify_cmd, P_STRING},
 #else
-	{"junk_learn_command", "bogofilter -s -I", &prefs_common.junk_learncmd,
-	 P_STRING},
-	{"nojunk_learn_command", "bogofilter -n -I",
+	{"junk_learn_command", "bogofilter -N -s -I",
+	 &prefs_common.junk_learncmd, P_STRING},
+	{"nojunk_learn_command", "bogofilter -n -S -I",
 	 &prefs_common.nojunk_learncmd, P_STRING},
 	{"junk_classify_command", "bogofilter -I",
 	 &prefs_common.junk_classify_cmd, P_STRING},
@@ -360,6 +435,8 @@ static PrefParam param[] = {
 
 	{"add_address_by_click", "FALSE", &prefs_common.add_address_by_click,
 	 P_BOOL},
+	{"enable_address_completion", "TRUE",
+	 &prefs_common.enable_address_completion, P_BOOL},
 
 	{"confirm_on_exit", "FALSE", &prefs_common.confirm_on_exit, P_BOOL},
 	{"clean_trash_on_exit", "FALSE", &prefs_common.clean_on_exit, P_BOOL},
@@ -397,6 +474,11 @@ static PrefParam param[] = {
 	{"ext_sendmail_cmd", DEFAULT_SENDMAIL_CMD, &prefs_common.extsend_cmd,
 	 P_STRING},
 
+	/* Update check */
+	{"auto_update_check", "TRUE", &prefs_common.auto_update_check, P_BOOL},
+	{"use_http_proxy", "FALSE", &prefs_common.use_http_proxy, P_BOOL},
+	{"http_proxy_host", NULL, &prefs_common.http_proxy_host, P_STRING},
+
 	/* Advanced */
 	{"strict_cache_check", "FALSE", &prefs_common.strict_cache_check,
 	 P_BOOL},
@@ -537,3 +619,36 @@ void prefs_common_junk_filter_list_set(void)
 	rule = prefs_common_junk_filter_rule_create(TRUE);
 	prefs_common.manual_junk_fltlist = g_slist_append(NULL, rule);
 }
+
+void prefs_common_junk_folder_rename_path(const gchar *old_path,
+					  const gchar *new_path)
+{
+	gint len;
+	gchar *base;
+	gchar *dest_path;
+
+	g_return_if_fail(old_path != NULL);
+	g_return_if_fail(new_path != NULL);
+
+	if (!prefs_common.junk_folder)
+		return;
+
+	len = strlen(old_path);
+
+	if (!strncmp(old_path, prefs_common.junk_folder, len)) {
+		base = prefs_common.junk_folder + len;
+		if (*base != '/' && *base != '\0')
+			return;
+		while (*base == '/') base++;
+		if (*base == '\0')
+			dest_path = g_strdup(new_path);
+		else
+			dest_path = g_strconcat(new_path, "/", base, NULL);
+		debug_print("prefs_common_junk_folder_rename_path(): "
+			    "renaming %s -> %s\n", prefs_common.junk_folder,
+			    dest_path);
+		g_free(prefs_common.junk_folder);
+		prefs_common.junk_folder = dest_path;
+		prefs_common_junk_filter_list_set();
+	}
+}
