@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,7 +67,9 @@
 #include "account_dialog.h"
 #include "folder.h"
 #include "inc.h"
+#include "send_message.h"
 #include "virtual.h"
+#include "plugin.h"
 
 enum
 {
@@ -176,6 +178,9 @@ static void folderview_update_summary_cb(FolderView	*folderview,
 static void folderview_mark_all_read_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
+static void folderview_send_queue_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
 
 static void folderview_new_folder_cb	(FolderView	*folderview,
 					 guint		 action,
@@ -257,6 +262,7 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Mar_k all read"),		NULL, folderview_mark_all_read_cb, 0, NULL},
+	{N_("/Send _queued messages"),	NULL, folderview_send_queue_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/Ed_it search condition..."),
@@ -281,6 +287,7 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Update summary"),	NULL, folderview_update_summary_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Mar_k all read"),		NULL, folderview_mark_all_read_cb, 0, NULL},
+	{N_("/Send _queued messages"),	NULL, folderview_send_queue_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/Ed_it search condition..."),
@@ -366,7 +373,7 @@ FolderView *folderview_create(void)
 
 	/* create folder icon + name column */
 	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_spacing(column, 2);
+	gtk_tree_view_column_set_spacing(column, 1);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width
 		(column, prefs_common.folder_col_folder);
@@ -540,13 +547,13 @@ void folderview_init(FolderView *folderview)
 
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_INBOX, &inbox_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_OUTBOX, &outbox_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_CLOSE, &folder_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_OPEN, &folderopen_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DIR_NOSELECT,
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_CLOSE, &folder_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_OPEN, &folderopen_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_NOSELECT,
 			 &foldernoselect_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_DRAFT, &draft_pixbuf);
 	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_TRASH, &trash_pixbuf);
-	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_GROUP, &virtual_pixbuf);
+	stock_pixbuf_gdk(treeview, STOCK_PIXMAP_FOLDER_SEARCH, &virtual_pixbuf);
 }
 
 void folderview_reflect_prefs(FolderView *folderview)
@@ -604,13 +611,13 @@ static void folderview_set_columns(FolderView *folderview)
 
 	column = gtk_tree_view_get_column(treeview, COL_NEW);
 	gtk_tree_view_column_set_visible
-		(column, prefs_common.display_folder_num_columns);
+		(column, prefs_common.folder_col_visible[COL_NEW]);
 	column = gtk_tree_view_get_column(treeview, COL_UNREAD);
 	gtk_tree_view_column_set_visible
-		(column, prefs_common.display_folder_num_columns);
+		(column, prefs_common.folder_col_visible[COL_UNREAD]);
 	column = gtk_tree_view_get_column(treeview, COL_TOTAL);
 	gtk_tree_view_column_set_visible
-		(column, prefs_common.display_folder_num_columns);
+		(column, prefs_common.folder_col_visible[COL_TOTAL]);
 }
 
 void folderview_select(FolderView *folderview, FolderItem *item)
@@ -905,6 +912,8 @@ static GtkWidget *label_window_create(const gchar *str)
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
 	manage_window_set_transient(GTK_WINDOW(window));
+	g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(gtk_true),
+			 NULL);
 
 	label = gtk_label_new(str);
 	gtk_container_add(GTK_CONTAINER(window), label);
@@ -1619,6 +1628,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	gboolean update_tree     = FALSE;
 	gboolean update_summary  = FALSE;
 	gboolean mark_all_read   = FALSE;
+	gboolean send_queue      = FALSE;
 	gboolean rescan_tree     = FALSE;
 	gboolean remove_tree     = FALSE;
 	gboolean search_folder   = FALSE;
@@ -1663,6 +1673,9 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 			} else if (item->stype == F_TRASH) {
 				if (item->total > 0)
 					empty_trash = TRUE;
+			} else if (item->stype == F_QUEUE) {
+				if (item->total > 0)
+					send_queue = TRUE;
 			}
 		} else if (FOLDER_TYPE(folder) == F_NEWS) {
 			if (item->parent != NULL)
@@ -1674,7 +1687,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 		}
 		if (FOLDER_TYPE(folder) == F_IMAP ||
 		    FOLDER_TYPE(folder) == F_NEWS) {
-			if (item->parent != NULL && item->no_select == FALSE &&
+			if (item->no_select == FALSE &&
 			    item->stype != F_VIRTUAL)
 				download_msg = TRUE;
 		}
@@ -1739,6 +1752,7 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 	SET_SENS(ifactory, "/Rebuild folder tree", rescan_tree);
 	SET_SENS(ifactory, "/Update summary", update_summary);
 	SET_SENS(ifactory, "/Mark all read", mark_all_read);
+	SET_SENS(ifactory, "/Send queued messages", send_queue);
 	SET_SENS(ifactory, "/Search messages...", search_folder);
 	SET_SENS(ifactory, "/Edit search condition...", search_folder);
 	SET_SENS(ifactory, "/Properties...", folder_property);
@@ -1754,15 +1768,19 @@ static gboolean folderview_menu_popup(FolderView *folderview,
 			       item->stype == F_VIRTUAL);
 	}
 
+	SET_VISIBILITY(ifactory, "/Mark all read", item->stype != F_QUEUE);
+	SET_VISIBILITY(ifactory, "/Send queued messages",
+		       item->stype == F_QUEUE);
 	SET_VISIBILITY(ifactory, "/Search messages...",
 		       item->stype != F_VIRTUAL);
 	SET_VISIBILITY(ifactory, "/Edit search condition...",
 		       item->stype == F_VIRTUAL);
 
-
 #undef SET_SENS
 #undef SET_VISIBILITY
 
+	syl_plugin_signal_emit("folderview-menu-popup", ifactory);
+
 	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
 		       event->button, event->time);
 
@@ -1874,7 +1892,9 @@ static gboolean folderview_focus_idle_func(gpointer data)
 {
 	FolderView *folderview = (FolderView *)data;
 
+	gdk_threads_enter();
 	GTK_WIDGET_SET_FLAGS(folderview->treeview, GTK_CAN_FOCUS);
+	gdk_threads_leave();
 
 	return FALSE;
 }
@@ -2085,6 +2105,7 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 {
 	MainWindow *mainwin = folderview->mainwin;
 	FolderItem *item;
+	gint ret = 0;
 
 	item = folderview_get_selected_item(folderview);
 	if (!item)
@@ -2094,6 +2115,22 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 
 	g_return_if_fail(item->folder != NULL);
 
+	if (item->parent == NULL) {
+		gchar *name, *msg;
+
+		name = trim_string(item->name, 32);
+		msg = g_strdup_printf(_("Download all messages under '%s' ?"),
+				      name);
+		g_free(name);
+		if (alertpanel(_("Download all messages"), msg,
+			       GTK_STOCK_YES, GTK_STOCK_NO, NULL)
+		    != G_ALERTDEFAULT) {
+			g_free(msg);
+			return;
+		}
+		g_free(msg);
+	}
+
 	if (!main_window_toggle_online_if_offline(folderview->mainwin))
 		return;
 
@@ -2104,13 +2141,40 @@ static void folderview_download_cb(FolderView *folderview, guint action,
 	main_window_progress_on(mainwin);
 	GTK_EVENTS_FLUSH();
 	folder_set_ui_func(item->folder, folderview_download_func, NULL);
-	if (folder_item_fetch_all_msg(item) < 0) {
+
+	if (item->parent == NULL) {
+		GtkTreeModel *model = GTK_TREE_MODEL(folderview->store);
+		GtkTreeIter iter;
+		gboolean valid;
+		FolderItem *cur_item;
+
+		valid = gtkut_tree_model_find_by_column_data
+			(model, &iter, NULL, COL_FOLDER_ITEM, item);
+		while ((valid = gtkut_tree_model_next(model, &iter)) == TRUE) {
+			cur_item = NULL;
+			gtk_tree_model_get(model, &iter, COL_FOLDER_ITEM,
+					   &cur_item, -1);
+			if (!cur_item || cur_item->folder != item->folder)
+				break;
+			if (!cur_item->no_select &&
+			    cur_item->stype != F_VIRTUAL &&
+			    cur_item->stype != F_TRASH) {
+				ret = folder_item_fetch_all_msg(cur_item);
+				if (ret < 0)
+					break;
+			}
+		}
+	} else
+		ret = folder_item_fetch_all_msg(item);
+
+	if (ret < 0) {
 		gchar *name;
 
 		name = trim_string(item->name, 32);
 		alertpanel_error(_("Error occurred while downloading messages in `%s'."), name);
 		g_free(name);
 	}
+
 	folder_set_ui_func(item->folder, NULL, NULL);
 	main_window_progress_off(mainwin);
 	gtk_widget_set_sensitive(folderview->treeview, TRUE);
@@ -2167,6 +2231,30 @@ static void folderview_mark_all_read_cb(FolderView *folderview, guint action,
 	}
 }
 
+static void folderview_send_queue_cb(FolderView *folderview, guint action,
+				     GtkWidget *widget)
+{
+	FolderItem *item;
+	gint ret;
+
+	if (!main_window_toggle_online_if_offline(folderview->mainwin))
+		return;
+
+	item = folderview_get_selected_item(folderview);
+	if (!item || item->stype != F_QUEUE)
+		return;
+
+	ret = send_message_queue_all(item, prefs_common.savemsg,
+				     prefs_common.filter_sent);
+	statusbar_pop_all();
+	if (ret > 0)
+		folder_item_scan(item);
+
+	folderview_update_item(item, TRUE);
+	main_window_set_menu_sensitive(folderview->mainwin);
+	main_window_set_toolbar_sensitive(folderview->mainwin);
+}
+
 static void folderview_new_folder_cb(FolderView *folderview, guint action,
 				     GtkWidget *widget)
 {
@@ -2200,6 +2288,16 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 	if (!new_folder) return;
 	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
 
+#ifdef G_OS_WIN32
+	p = strpbrk(new_folder, "\\/:*?\"<>|");
+	if ((p && FOLDER_TYPE(item->folder) != F_IMAP) || (p && *p != '/') ||
+	    (p && *p == '/' &&
+	     FOLDER_TYPE(item->folder) == F_IMAP && *(p + 1) != '\0')) {
+		alertpanel_error(_("`%c' can't be included in folder name."),
+				 *p);
+		return;
+	}
+#else
 	p = strchr(new_folder, G_DIR_SEPARATOR);
 	if ((p && FOLDER_TYPE(item->folder) != F_IMAP) ||
 	    (p && FOLDER_TYPE(item->folder) == F_IMAP && *(p + 1) != '\0')) {
@@ -2207,6 +2305,7 @@ static void folderview_new_folder_cb(FolderView *folderview, guint action,
 				 G_DIR_SEPARATOR);
 		return;
 	}
+#endif
 
 	name = trim_string(new_folder, 32);
 	AUTORELEASE_STR(name, {g_free(name); return;});
@@ -2299,10 +2398,13 @@ static void folderview_rename_folder_cb(FolderView *folderview, guint action,
 		return;
 	}
 
-	if (folder_get_default_folder() == item->folder)
+	if (folder_get_default_folder() == item->folder) {
 		filter_list_rename_path(old_path, item->path);
+		prefs_common_junk_folder_rename_path(old_path, item->path);
+	}
 	new_id = folder_item_get_identifier(item);
 	filter_list_rename_path(old_id, new_id);
+	prefs_common_junk_folder_rename_path(old_id, new_id);
 	g_free(old_id);
 	g_free(new_id);
 
@@ -2369,10 +2471,13 @@ static void folderview_move_folder_cb(FolderView *folderview, guint action,
 		return;
 	}
 
-	if (folder_get_default_folder() == item->folder)
+	if (folder_get_default_folder() == item->folder) {
 		filter_list_rename_path(old_path, item->path);
+		prefs_common_junk_folder_rename_path(old_path, item->path);
+	}
 	new_id = folder_item_get_identifier(item);
 	filter_list_rename_path(old_id, new_id);
+	prefs_common_junk_folder_rename_path(old_id, new_id);
 	g_free(new_id);
 	g_free(old_id);
 	g_free(old_path);
@@ -2900,6 +3005,9 @@ static gint auto_expand_timeout(gpointer data)
 	FolderView *folderview = data;
 	GtkTreeView *treeview = GTK_TREE_VIEW(folderview->treeview);
 	GtkTreePath *path = NULL;
+	gint ret;
+
+	gdk_threads_enter();
 
 	gtk_tree_view_get_drag_dest_row(treeview, &path, NULL);
 
@@ -2908,9 +3016,13 @@ static gint auto_expand_timeout(gpointer data)
 		gtk_tree_path_free(path);
 		folderview->expand_timeout = 0;
 
-		return FALSE;
+		ret = FALSE;
 	} else
-		return TRUE;
+		ret = TRUE;
+
+	gdk_threads_leave();
+
+	return ret;
 }
 
 static void remove_auto_expand_timeout(FolderView *folderview)
@@ -2925,9 +3037,13 @@ static gint auto_scroll_timeout(gpointer data)
 {
 	FolderView *folderview = data;
 
+	gdk_threads_enter();
+
 	gtkut_tree_view_vertical_autoscroll
 		(GTK_TREE_VIEW(folderview->treeview));
 
+	gdk_threads_leave();
+
 	return TRUE;
 }
 
