@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,6 +78,7 @@ struct _MainWindow
 	GtkWidget *toolbar;
 	GtkWidget *get_btn;
 	GtkWidget *getall_btn;
+	GtkWidget *rpop3_btn;
 	GtkWidget *send_btn;
 	GtkWidget *compose_btn;
 	GtkWidget *reply_btn;
@@ -151,7 +152,8 @@ MainWindow *main_window_get		(void);
 GtkWidget *main_window_get_folder_window	(MainWindow	*mainwin);
 GtkWidget *main_window_get_message_window	(MainWindow	*mainwin);
 
-void main_window_separation_change	(MainWindow	*mainwin,
+void main_window_change_layout		(MainWindow	*mainwin,
+					 LayoutType	 layout,
 					 SeparateType	 type);
 
 void main_window_toggle_message_view	(MainWindow	*mainwin);
