@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,7 +81,8 @@ static struct FilterEditHeaderListDialog {
 
 static void prefs_filter_edit_create		(void);
 static void prefs_filter_edit_clear		(void);
-static void prefs_filter_edit_rule_to_dialog	(FilterRule	*rule);
+static void prefs_filter_edit_rule_to_dialog	(FilterRule	*rule,
+						 const gchar	*default_name);
 static void prefs_filter_edit_update_header_list(FilterCondEdit	*cond_list);
 
 static void prefs_filter_edit_set_action_hbox_menu_sensitive
@@ -128,6 +129,8 @@ static void prefs_filter_edit_cancel		(void);
 
 static void prefs_filter_cond_activated_cb	(GtkWidget	*widget,
 						 gpointer	 data);
+static void prefs_filter_match_activated_cb	(GtkWidget	*widget,
+						 gpointer	 data);
 static void prefs_filter_action_activated_cb	(GtkWidget	*widget,
 						 gpointer	 data);
 
@@ -144,7 +147,8 @@ static void prefs_filter_action_add_cb		(GtkWidget	*widget,
 						 gpointer	 data);
 
 
-FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header)
+FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header,
+				   const gchar *key)
 {
 	static gboolean lock = FALSE;
 	FilterRule *new_rule;
@@ -160,7 +164,7 @@ FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header)
 	manage_window_set_transient(GTK_WINDOW(rule_edit_window.window));
 
 	prefs_filter_edit_set_header_list(&rule_edit_window.cond_edit, rule);
-	prefs_filter_edit_rule_to_dialog(rule);
+	prefs_filter_edit_rule_to_dialog(rule, key);
 	if (header)
 		prefs_filter_edit_activate_cond_header
 			(&rule_edit_window.cond_edit, header);
@@ -345,7 +349,7 @@ void prefs_filter_edit_clear_cond_edit(FilterCondEdit *cond_edit)
 		prefs_filter_edit_remove_cond_hbox(cond_edit, hbox);
 	}
 
-	g_slist_free(cond_edit->hdr_list);
+	procheader_header_list_destroy(cond_edit->hdr_list);
 	cond_edit->hdr_list = NULL;
 	procheader_header_list_destroy(cond_edit->rule_hdr_list);
 	cond_edit->rule_hdr_list = NULL;
@@ -362,7 +366,8 @@ static void prefs_filter_edit_clear(void)
 	}
 }
 
-static void prefs_filter_edit_rule_to_dialog(FilterRule *rule)
+static void prefs_filter_edit_rule_to_dialog(FilterRule *rule,
+					     const gchar *default_name)
 {
 	gint index = 0;
 	static gint count = 1;
@@ -370,6 +375,9 @@ static void prefs_filter_edit_rule_to_dialog(FilterRule *rule)
 	if (rule && rule->name)
 		gtk_entry_set_text(GTK_ENTRY(rule_edit_window.name_entry),
 				   rule->name);
+	else if (default_name)
+		gtk_entry_set_text(GTK_ENTRY(rule_edit_window.name_entry),
+				   default_name);
 	else {
 		gchar rule_name[32];
 		g_snprintf(rule_name, sizeof(rule_name), "Rule %d", count++);
@@ -407,7 +415,7 @@ void prefs_filter_edit_set_header_list(FilterCondEdit *cond_edit,
 	GSList *cur;
 	FilterCond *cond;
 
-	g_slist_free(cond_edit->hdr_list);
+	procheader_header_list_destroy(cond_edit->hdr_list);
 	cond_edit->hdr_list = NULL;
 	procheader_header_list_destroy(cond_edit->rule_hdr_list);
 	cond_edit->rule_hdr_list = NULL;
@@ -429,21 +437,22 @@ void prefs_filter_edit_set_header_list(FilterCondEdit *cond_edit,
 	}
 
 	cond_edit->rule_hdr_list = rule_hdr_list;
-	cond_edit->hdr_list = procheader_merge_header_list(list, rule_hdr_list);
+	cond_edit->hdr_list = procheader_merge_header_list_dup(list,
+							       rule_hdr_list);
+	procheader_header_list_destroy(list);
 }
 
 static void prefs_filter_edit_update_header_list(FilterCondEdit *cond_edit)
 {
 	GSList *list;
 
-	g_slist_free(cond_edit->hdr_list);
+	procheader_header_list_destroy(cond_edit->hdr_list);
 	cond_edit->hdr_list = NULL;
 
 	list = prefs_filter_get_header_list();
-	cond_edit->hdr_list = list;
-
-	cond_edit->hdr_list =
-		procheader_merge_header_list(list, cond_edit->rule_hdr_list);
+	cond_edit->hdr_list = procheader_merge_header_list_dup
+		(list, cond_edit->rule_hdr_list);
+	procheader_header_list_destroy(list);
 }
 
 CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
@@ -465,6 +474,8 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	GtkWidget *add_btn;
 	GtkWidget *del_img;
 	GtkWidget *add_img;
+	GtkWidget *match_menu_in_addr;
+	GtkWidget *match_menu_not_in_addr;
 
 	cond_hbox = g_new0(CondHBox, 1);
 
@@ -511,24 +522,31 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	gtk_widget_show(match_type_optmenu);
 	gtk_box_pack_start(GTK_BOX(hbox), match_type_optmenu, FALSE, FALSE, 0);
 
+#define MATCH_MENUITEM_ADD(str, action)					    \
+{									    \
+	MENUITEM_ADD(menu, menuitem, str, action);			    \
+	g_signal_connect(G_OBJECT(menuitem), "activate",		    \
+			 G_CALLBACK(prefs_filter_match_activated_cb),	    \
+			 cond_hbox);					    \
+}
 	menu = gtk_menu_new();
 	gtk_widget_show(menu);
-	MENUITEM_ADD(menu, menuitem, _("contains"),
-		     PF_MATCH_CONTAIN);
-	MENUITEM_ADD(menu, menuitem, _("doesn't contain"),
-		     PF_MATCH_NOT_CONTAIN);
-	MENUITEM_ADD(menu, menuitem, _("is"),
-		     PF_MATCH_EQUAL);
-	MENUITEM_ADD(menu, menuitem, _("is not"),
-		     PF_MATCH_NOT_EQUAL);
+	MATCH_MENUITEM_ADD(_("contains"),               PF_MATCH_CONTAIN);
+	MATCH_MENUITEM_ADD(_("doesn't contain"),        PF_MATCH_NOT_CONTAIN);
+	MATCH_MENUITEM_ADD(_("is"),                     PF_MATCH_EQUAL);
+	MATCH_MENUITEM_ADD(_("is not"),                 PF_MATCH_NOT_EQUAL);
 #if defined(USE_ONIGURUMA) || defined(HAVE_REGCOMP)
-	MENUITEM_ADD(menu, menuitem, _("match to regex"),
-		     PF_MATCH_REGEX);
-	MENUITEM_ADD(menu, menuitem, _("doesn't match to regex"),
-		     PF_MATCH_NOT_REGEX);
+	MATCH_MENUITEM_ADD(_("match to regex"),         PF_MATCH_REGEX);
+	MATCH_MENUITEM_ADD(_("doesn't match to regex"), PF_MATCH_NOT_REGEX);
 #endif
+	MATCH_MENUITEM_ADD(_("is in addressbook"),      PF_MATCH_IN_ADDRESSBOOK);
+	match_menu_in_addr = menuitem;
+	MATCH_MENUITEM_ADD(_("is not in addressbook"),  PF_MATCH_NOT_IN_ADDRESSBOOK);
+	match_menu_not_in_addr = menuitem;
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(match_type_optmenu), menu);
 
+#undef MATCH_MENUITEM_ADD
+
 	size_match_optmenu = gtk_option_menu_new();
 	gtk_widget_show(size_match_optmenu);
 	gtk_box_pack_start(GTK_BOX(hbox), size_match_optmenu, FALSE, FALSE, 0);
@@ -605,6 +623,8 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	cond_hbox->key_entry = key_entry;
 	cond_hbox->spin_btn = spin_btn;
 	cond_hbox->label = label;
+	cond_hbox->match_menu_in_addr = match_menu_in_addr;
+	cond_hbox->match_menu_not_in_addr = match_menu_not_in_addr;
 	cond_hbox->del_btn = del_btn;
 	cond_hbox->add_btn = add_btn;
 	cond_hbox->cur_type = PF_COND_HEADER;
@@ -695,7 +715,7 @@ ActionHBox *prefs_filter_edit_action_hbox_create(void)
 	gtk_box_pack_start(GTK_BOX(hbox), folder_entry, TRUE, TRUE, 0);
 
 	folder_img = stock_pixbuf_widget(rule_edit_window.window,
-					 STOCK_PIXMAP_DIR_OPEN);
+					 STOCK_PIXMAP_FOLDER_OPEN);
 	gtk_widget_show(folder_img);
 
 	folder_sel_btn = gtk_button_new();
@@ -834,6 +854,12 @@ void prefs_filter_edit_cond_hbox_set(CondHBox *hbox, FilterCond *cond)
 			else
 				match_type = PF_MATCH_REGEX;
 			break;
+		case FLT_IN_ADDRESSBOOK:
+			if (FLT_IS_NOT_MATCH(cond->match_flag))
+				match_type = PF_MATCH_NOT_IN_ADDRESSBOOK;
+			else
+				match_type = PF_MATCH_IN_ADDRESSBOOK;
+			break;
 		}
 		break;
 	case FLT_COND_UNREAD:
@@ -888,6 +914,10 @@ void prefs_filter_edit_cond_hbox_set(CondHBox *hbox, FilterCond *cond)
 		gtk_option_menu_set_history
 			(GTK_OPTION_MENU(hbox->status_match_optmenu),
 			 status_type);
+
+	if (match_type == PF_MATCH_IN_ADDRESSBOOK ||
+	    match_type == PF_MATCH_NOT_IN_ADDRESSBOOK)
+		gtk_widget_hide(hbox->key_entry);
 }
 
 void prefs_filter_edit_action_hbox_set(ActionHBox *hbox, FilterAction *action)
@@ -970,6 +1000,8 @@ void prefs_filter_edit_cond_hbox_select(CondHBox *hbox, CondMenuType type,
 
 void prefs_filter_edit_set_cond_hbox_widgets(CondHBox *hbox, CondMenuType type)
 {
+	MatchMenuType match_type;
+
 	switch (type) {
 	case PF_COND_HEADER:
 	case PF_COND_TO_OR_CC:
@@ -979,9 +1011,27 @@ void prefs_filter_edit_set_cond_hbox_widgets(CondHBox *hbox, CondMenuType type)
 		gtk_widget_hide(hbox->size_match_optmenu);
 		gtk_widget_hide(hbox->age_match_optmenu);
 		gtk_widget_hide(hbox->status_match_optmenu);
-		gtk_widget_show(hbox->key_entry);
+		match_type = menu_get_option_menu_active_index
+			(GTK_OPTION_MENU(hbox->match_type_optmenu));
+		if (match_type == PF_MATCH_IN_ADDRESSBOOK ||
+		    match_type == PF_MATCH_NOT_IN_ADDRESSBOOK)
+			gtk_widget_hide(hbox->key_entry);
+		else
+			gtk_widget_show(hbox->key_entry);
 		gtk_widget_hide(hbox->spin_btn);
 		gtk_widget_hide(hbox->label);
+		if (type == PF_COND_HEADER || type == PF_COND_TO_OR_CC) {
+			gtk_widget_show(hbox->match_menu_in_addr);
+			gtk_widget_show(hbox->match_menu_not_in_addr);
+		} else {
+			gtk_widget_hide(hbox->match_menu_in_addr);
+			gtk_widget_hide(hbox->match_menu_not_in_addr);
+			if (match_type == PF_MATCH_IN_ADDRESSBOOK ||
+			    match_type == PF_MATCH_NOT_IN_ADDRESSBOOK) {
+				gtk_option_menu_set_history(GTK_OPTION_MENU(hbox->match_type_optmenu), 0);
+				gtk_widget_show(hbox->key_entry);
+			}
+		}
 		break;
 	case PF_COND_CMD_TEST:
 		gtk_widget_hide(hbox->match_type_optmenu);
@@ -1534,6 +1584,7 @@ static void prefs_filter_edit_edit_header_list_dialog_create(void)
 	gtk_container_add(GTK_CONTAINER(scrwin), clist);
 	gtk_clist_set_column_width(GTK_CLIST(clist), 0, 80);
 	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
+	gtkut_clist_set_redraw(GTK_CLIST(clist));
 	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist)->column[0].button,
 			       GTK_CAN_FOCUS);
 
@@ -1633,6 +1684,7 @@ static void prefs_filter_edit_edit_header_list(FilterCondEdit *cond_edit)
 			CondHBox *hbox = (CondHBox *)cur->data;
 			prefs_filter_edit_set_cond_header_menu(cond_edit, hbox);
 		}
+		prefs_filter_write_user_header_list();
 	}
 
 	gtk_widget_destroy(edit_header_list_dialog.window);
@@ -1694,6 +1746,13 @@ FilterCond *prefs_filter_edit_cond_hbox_to_cond(CondHBox *hbox,
 		match_type = FLT_REGEX;
 		match_flag |= FLT_NOT_MATCH;
 		break;
+	case PF_MATCH_IN_ADDRESSBOOK:
+		match_type = FLT_IN_ADDRESSBOOK;
+		break;
+	case PF_MATCH_NOT_IN_ADDRESSBOOK:
+		match_type = FLT_IN_ADDRESSBOOK;
+		match_flag |= FLT_NOT_MATCH;
+		break;
 	default:
 		break;
 	}
@@ -2026,6 +2085,21 @@ static void prefs_filter_cond_activated_cb(GtkWidget *widget, gpointer data)
 	}
 }
 
+static void prefs_filter_match_activated_cb(GtkWidget *widget, gpointer data)
+{
+	CondHBox *hbox = (CondHBox *)data;
+	GtkWidget *cond_type_menuitem;
+	CondMenuType cond_menu_type;
+
+	cond_type_menuitem = gtk_menu_get_active
+		(GTK_MENU(gtk_option_menu_get_menu
+			(GTK_OPTION_MENU(hbox->cond_type_optmenu))));
+	cond_menu_type = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(cond_type_menuitem), MENU_VAL_ID));
+
+	prefs_filter_edit_set_cond_hbox_widgets(hbox, cond_menu_type);
+}
+
 static void prefs_filter_action_activated_cb(GtkWidget *widget, gpointer data)
 {
 	ActionHBox *hbox = (ActionHBox *)data;
