@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,9 +32,9 @@ typedef enum
 	STOCK_PIXMAP_COMPLETE,
 	STOCK_PIXMAP_CONTINUE,
 	STOCK_PIXMAP_DELETED,
-	STOCK_PIXMAP_DIR_CLOSE,
-	STOCK_PIXMAP_DIR_OPEN,
-	STOCK_PIXMAP_DIR_NOSELECT,
+	STOCK_PIXMAP_FOLDER_CLOSE,
+	STOCK_PIXMAP_FOLDER_OPEN,
+	STOCK_PIXMAP_FOLDER_NOSELECT,
 	STOCK_PIXMAP_ERROR,
 	STOCK_PIXMAP_FORWARDED,
 	STOCK_PIXMAP_GROUP,
@@ -69,17 +69,28 @@ typedef enum
 	STOCK_PIXMAP_ADDRESS_BOOK,
 	STOCK_PIXMAP_DELETE,
 	STOCK_PIXMAP_SPAM,
+	STOCK_PIXMAP_NOTSPAM,
 	STOCK_PIXMAP_SIGN,
 	STOCK_PIXMAP_SYLPHEED,
+	STOCK_PIXMAP_SYLPHEED_SMALL,
+	STOCK_PIXMAP_SYLPHEED_32,
+	STOCK_PIXMAP_SYLPHEED_NEWMAIL,
+	STOCK_PIXMAP_SYLPHEED_NEWMAIL_SMALL,
 	STOCK_PIXMAP_SYLPHEED_LOGO,
+	STOCK_PIXMAP_PERSON,
+	STOCK_PIXMAP_FOLDER_SEARCH,
 
 	N_STOCK_PIXMAPS
 } StockPixmap;
 
-GtkWidget *stock_pixmap_widget	(GtkWidget	 *window,
-				 StockPixmap	  icon);
 GtkWidget *stock_pixbuf_widget	(GtkWidget	 *window,
 				 StockPixmap	  icon);
+
+GtkWidget *stock_pixbuf_widget_scale	(GtkWidget	*window,
+					 StockPixmap	 icon,
+					 gint		 width,
+					 gint		 height);
+
 gint stock_pixmap_gdk		(GtkWidget	 *window,
 				 StockPixmap	  icon,
 				 GdkPixmap	**pixmap,
