@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2004 Hiroyuki Yamamoto
+ * Copyright (C) 2001-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@ struct _Template {
 	gchar *name;
 	gchar *to;
 	gchar *cc;
+	gchar *bcc;
+	gchar *replyto;
 	gchar *subject;
 	gchar *value;
 };
